// ZenLib::Utils - Very small utilities
// Copyright (C) 2002-2004 Jrme Martinez, Zen@MediaArea.net
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 3. This notice may not be removed or altered from any source distribution.
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ZenUtils
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Version 0.1.0
// -------------
// BigEndianToInt    (8-64 bits)
// LittleEndianToInt (8-64 bits)
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// 0.1.0 (Zen@MediaArea.net)
// BigEndianToInt    (8-64 bits)
// LittleEndianToInt (8-64 bits)
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//---------------------------------------------------------------------------
#ifndef ZenUtilsH
#define ZenUtilsH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#include <ZenLib/Conf.h>
//---------------------------------------------------------------------------

namespace ZenLib
{

//***************************************************************************
// Integer transformations
//***************************************************************************

//---------------------------------------------------------------------------
//Little Endians
int8s  LittleEndian2int8s  (const char* List);
int8u  LittleEndian2int8u  (const char* List);
int16s LittleEndian2int16s (const char* List);
int16u LittleEndian2int16u (const char* List);
int32s LittleEndian2int32s (const char* List);
int32u LittleEndian2int32u (const char* List);
#if (MAXTYPE_INT >= 64)
int64s LittleEndian2int64s (const char* List);
int64u LittleEndian2int64u (const char* List);
#endif

//---------------------------------------------------------------------------
//Big Endians
int8s  BigEndian2int8s     (const char* List);
int8u  BigEndian2int8u     (const char* List);
int16s BigEndian2int16s    (const char* List);
int16u BigEndian2int16u    (const char* List);
int32s BigEndian2int32s    (const char* List);
int32u BigEndian2int32u    (const char* List);
#if (MAXTYPE_INT >= 64)
int64s BigEndian2int64s    (const char* List);
int64u BigEndian2int64u    (const char* List);
#endif

//---------------------------------------------------------------------------
// int32 - int64
#if (MAXTYPE_INT >= 64)
    int64s int32s_int64s (                int32s  High, int32u  Low);
    int64u int32u_int64u (                int32u  High, int32u  Low);
    void   int32s_int64s (int64s &BigInt, int32s  High, int32u  Low);
    void   int32u_int64u (int64s &BigInt, int32u  High, int32u  Low);
    void   int64s_int32s (int64s  BigInt, int32s &High, int32u &Low);
    void   int64u_int32u (int64u  BigInt, int32u &High, int32u &Low);
#endif

//---------------------------------------------------------------------------
// Floats and ints
int32s float32_int32s  (float32 F, bool Rounded=true);
int64s float32_int64s  (float32 F, bool Rounded=true);
int32s float64_int32s  (float64 F, bool Rounded=true);
int64s float64_int64s  (float64 F, bool Rounded=true);

// These functions are used because MSVC6 isn't able to convert an unsigned int64 to a floating-point value, and I couldn't think of a cleaner way to handle it.
#if _MSC_VER <= 1200
inline float32 int64u_float32  (int64u v) { return static_cast<float32>(static_cast<int64s>(v>>1))*2.0f + static_cast<float32>(static_cast<int64s>(v & 1)); }
inline float64 int64u_float64  (int64u v) { return static_cast<float64>(static_cast<int64s>(v>>1))*2.0f + static_cast<float32>(static_cast<int64s>(v & 1)); }
#else
#pragma
#pragma warning( disable : 4244 )
inline float32 int64u_float32  (int64u v) { return v; }
inline float64 int64u_float64  (int64u v) { return v; }
#pragma warning( default : 4244 )
#endif

} //namespace ZenLib


/*
#include <list>
#include <stdio.h>


      typedef struct {
	      DWORD	address;
	      DWORD	size;
	      char	file[64];
	      DWORD	line;
      } ALLOC_INFO;

      typedef std::list<ALLOC_INFO*> AllocList;

      AllocList *allocList;

      void AddTrack(DWORD addr,  DWORD asize,  const char *fname, DWORD lnum)
      {
	      ALLOC_INFO *info;

	      if(!allocList) {
		      allocList = new(AllocList);
	      }

	      info = new(ALLOC_INFO);
	      info->address = addr;
	      strncpy(info->file, fname, 63);
	      info->line = lnum;
	      info->size = asize;
	      allocList->insert(allocList->begin(), info);
      };

      void RemoveTrack(DWORD addr)
      {
	      AllocList::iterator i;

	      if(!allocList)
		      return;
	      for(i = allocList->begin(); i != allocList->end(); i++)
	      {
		      if((*i)->address == addr)
		      {
			      allocList->remove((*i));
			      break;
		      }
	      }
      };

      #ifdef _DEBUG
      inline void * __cdecl operator new(unsigned int size,
                                         const char *file, int line)
      {
	      void *ptr = (void *)malloc(size);
	      AddTrack((DWORD)ptr, size, file, line);
	      return(ptr);
      };
      inline void * __cdecl operator new[](unsigned int size,
                                         const char *file, int line)
      {
	      void *ptr = (void *)malloc(size);
	      AddTrack((DWORD)ptr, size, file, line);
	      return(ptr);
      };
      inline void __cdecl operator delete(void *p)
      {
	      RemoveTrack((DWORD)p);
	      free(p);
      };
      inline void __cdecl operator delete[](void *p)
      {
	      RemoveTrack((DWORD)p);
	      free(p);
      };
      #define DEBUG_NEW new(__FILE__, __LINE__)
      #else
      #define DEBUG_NEW new
      #endif
      #define new DEBUG_NEW


      void DumpUnfreed()
      {
	      AllocList::iterator i;
	      DWORD totalSize = 0;

            FILE *stream;
            stream = fopen("d:\\MemoryLeak.txt", "w+");


	      if(!allocList)
		      return;

	      for(i = allocList->begin(); i != allocList->end(); i++) {
		      fprintf(stream, "%-50s:\t\tLine %d,\t\tAddress %d\t%d unfreed\n",
			      (*i)->file, (*i)->line, (*i)->address, (*i)->size);
		      totalSize += (*i)->size;
	      }
	      fprintf(stream, "-----------------------------------------------------------\n");
	      fprintf(stream, "Total Unfreed: %d bytes\n", totalSize);
      };


*/
#endif
